;;  mrdead Mixins.  Each mixin implements a MrEd interface.

(module mrdead-mixins mzscheme
  (provide gtk-object-mixin
           area-mixin
           subarea-mixin
           window-mixin
           subwindow-mixin
           area-container-mixin
           area-container-window-mixin
           top-level-window-mixin
           panel-mixin
           horizontal-panel-mixin
           vertical-panel-mixin
           control-mixin
           button-mixin
           message-mixin)

  (require (lib "class.ss")
           "mrdead-interfaces.ss"
           "../gtk.ss"
           "../gobject.ss")
  
  (define gtk-object-mixin
    (mixin () (gtk-object<%>)
      ;the gtk object
      (init-field local-gtk-item)
      
      (define/public (set-real-gtk-object! item) 
        (set! local-gtk-item item))
      
      (define/public (get-real-gtk-object) 
        local-gtk-item)
      
      (super-new)))
  
  (define area-mixin
    (mixin (gtk-object<%>) (area<%>)
      ;; stubs
      (init-field (stretchable-height #f))
      (init-field (stretchable-width #f))
      (init-field (style '()))

      (define/public (get-parent) #f)
      
      (super-new)))

  
  (define window-mixin
    (mixin (area<%>) (window<%>)
      (inherit-field local-gtk-item)
      
      (define/public (show show?)
        (if show?
            (gtk-widget-show local-gtk-item)
            (gtk-widget-hide local-gtk-item)))
      
      (super-new)))
  
  ;; abstract
  (define area-container-mixin
    (mixin (area<%>) (area-container<%>)
      ;; stub
      (init-field [alignment '()])

      (define children '())
      
      (inherit get-real-gtk-object)
      
      (define/pubment (add-child child)
        (set! children (cons child children))
        (unless (inner #f add-child child)
          (gtk-container-add (get-real-gtk-object) (send child get-real-gtk-object))))
      
      (super-new)))
  
  (define area-container-window-mixin
    (mixin (area-container<%> window<%>) (area-container-window<%>)
      (super-new)))
 
  (define subarea-mixin
    (mixin (area<%>) (subarea<%>)
      (super-new)))
  
  ;; abstract
  (define subwindow-mixin
    (mixin (subarea<%> window<%>) (subwindow<%>)
      (init-field parent)

      (define/override (get-parent)
        parent)
      
      ;; initialize the libGTK reference
      (super-new)
      
      (send parent add-child this)))
  
  
  (define top-level-window-mixin
    (mixin (area-container-window<%>) (top-level-window<%>)
      (init-field label)
      (super-new [local-gtk-item (gtk-window-new 'toplevel)])
      (gtk-window-set-title local-gtk-item label)

      (define/pubment (on-close)
        (inner #f on-close))
      
      (inherit-field local-gtk-item)
      (connect-signal local-gtk-item "delete_event" (lambda () (on-close)))

      ))
  
  (define panel-mixin
    (mixin (area-container-window<%> subwindow<%>) (panel<%>)
      
      (super-new)))

  (define horizontal-panel-mixin
    (mixin (panel<%>) (horizontal-panel<%>)

      (inherit get-real-gtk-object)
      (define/augment (add-child child)
        (gtk-box-pack-start-defaults (get-real-gtk-object) (send child get-real-gtk-object)))
      
      (super-new [local-gtk-item (gtk-hbox-new #t 0)])
      
      (inherit show)
      (show #t)))

  (define vertical-panel-mixin
    (mixin (panel<%>) (vertical-panel<%>)

      (inherit get-real-gtk-object)
      (define/augment (add-child child)
        (gtk-box-pack-start-defaults (get-real-gtk-object) (send child get-real-gtk-object)))
      
      (super-new [local-gtk-item (gtk-vbox-new #t 0)])
      
      (inherit show)
      (show #t)))
  
  (define control-mixin
    (mixin (subwindow<%>) (control<%>)
      (super-new)))

  (define button-mixin
    (mixin (control<%>) (button<%>)
      (init-field label)
      (init-field callback)
      
      ;stubs
      
      ;(init-field (callback (lambda (x y) (display x) (display y))))

      (super-new [local-gtk-item (gtk-button-new-with-label label)])
      
      (inherit-field local-gtk-item)
      (connect-signal local-gtk-item "clicked" (lambda ()
                                                 (callback this 'clicked)))
      (inherit show)
      (show #t)))
    
  (define message-mixin
    (mixin (control<%>) (message<%>)
      (init-field label)

      (inherit-field local-gtk-item)
      
      (define/public (set-label new-text)
        (gtk-label-set-text local-gtk-item new-text))
      
      (define/public (get-label)
        (gtk-label-get-text local-gtk-item))
      
      (super-new [local-gtk-item (gtk-label-new label)])
      
      (inherit show)
      (show #t)))

  
;    (define gtk-primative-entry-mixin
;    (mixin (control<%>) (something<%>)
;      (init-field label)
;
;      (inherit-field local-gtk-item)
;      
;      (define/public (set-value new-text)
;        (gtk-entry-set-something local-gtk-item new-text))
;      
;      (define/public (get-value)
;        (gtk-entry-get-something local-gtk-item))
;      
;      (super-new [local-gtk-item (gtk-label-new label)])
;      
;      (inherit show)
;      (show #t)))
  
  )